@@ -32,6 +32,8 @@ class TrainingArguments(Mapping):
     callbacks: Optional[Union[List[Callback], Dict[str, Callback]]] = None
     fp16: Union[Dict, bool] = None
     catalyst_args: Dict = field(default_factory=lambda: {})
+    train_transforms: Callable = None
+    val_transforms: Callable = None
 
     def __getitem__(self, k):
         if hasattr(self, k):
@@ -73,7 +75,7 @@ class InferDictCallback(InferCallback):
             if self.__current_store is not None:
                 self.__current_store(loader_name=state.loader_name)
             return
-        dct = {key: value.detach().cpu().numpy() for key, value in dct.items()}
+        dct = {key: value.detach().cpu().numpy() for key, value in dct.items() if key != 'gt'}
         loader_name = state.loader_name
         targets = state.input['targets']
         self.update_storage(loader_name, dct, targets)
@@ -181,10 +183,11 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         kwargs['logdir'] = logdir
         interpretation_callback = self.create_interpretation_callback(**kwargs)
         infer_dict_callback = InferDictCallback()
-        replace_gather_callback = ReplaceGatherCallback(self.task_flow, infer_dict_callback)
         default_callbacks = OrderedDict([("interpretation", interpretation_callback),
-                                         ("inference", infer_dict_callback),
-                                         ("dataparallel_reducer", replace_gather_callback)])
+                                         ("inference", infer_dict_callback)])
+        if self.balance_dataparallel_memory:
+            replace_gather_callback = ReplaceGatherCallback(self.task_flow, infer_dict_callback)
+            default_callbacks["dataparallel_reducer"] = replace_gather_callback
         kwargs['callbacks'] = kwargs.get('callbacks', default_callbacks)
         kwargs['model'] = kwargs.get('model', self.model)
         store = kwargs.pop('store', True)
@@ -247,9 +250,9 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
             raise ValueError(f'You must supply either a `loaders` parameter, or give `train_test_val_indices` via'
                              f'constructor.')
         train_indices, test_indices, val_indices = self.train_test_val_indices
-        train_dataset = TransformedSubset(dataset, train_indices)
-        val_dataset = TransformedSubset(dataset, val_indices)
-        test_dataset = TransformedSubset(dataset, test_indices)
+        train_dataset = TransformedSubset(dataset, train_indices, sample_transforms=kwargs.get('train_transforms'))
+        val_dataset = TransformedSubset(dataset, val_indices, sample_transforms=kwargs.get('val_transforms'))
+        test_dataset = TransformedSubset(dataset, test_indices, sample_transforms=kwargs.get('val_transforms'))
 
         datasets = {
             'train': train_dataset,
@@ -312,6 +315,28 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         df.to_csv(self.project_dir / self.default_logdir / 'evaluation.csv', index=False)
         return df
 
+    def _run_batch(self, batch: Mapping[str, Any]) -> None:
+        """
+        Inner method to run train step on specified data batch,
+        with batch callbacks events.
+
+        Args:
+            batch (Mapping[str, Any]): dictionary with data batches
+                from DataLoader.
+        """
+        if isinstance(batch, dict):
+            self.batch_size = len(next(iter(batch.values())))
+        else:
+            self.batch_size = len(next(iter(batch[1].values())))
+        self.global_sample_step += self.batch_size
+        self.loader_sample_step += self.batch_size
+        batch = self._batch2device(batch, self.device)
+        self.input = batch
+
+        self._run_event("on_batch_start")
+        self._handle_batch(batch=batch)
+        self._run_event("on_batch_end")
+
 
 def split_already_done(df, project_dir):
     total_len = 0
